<?php
//
// CoreLogonSession.php
// Session login for NagVis based on session/cookies
//
// Copyright 2018 - Nagios Enterprises, LLC
//

class CoreLogonSession extends CoreLogonModule {
    public function check($printErr = true) {
        global $AUTH, $CORE;

        $cookieName = 'nagiosxi';
        $usernameVar = 'username';
        $userLevelVar = 'userlevel';
        $url = $_SERVER['REQUEST_URI'];

        // Redirect user if the cookie doesn't exist (they aren't logged in)
        if (!array_key_exists($cookieName, $_COOKIE)) {
            if (CONST_AJAX) {
                throw new NagVisException(l('Nagios XI: Not authenticated.'));
            }
            header("Location: ../../../nagiosxi/login.php?redirect=".urlencode($url));
        }

        // Read the cookie
        session_id($_COOKIE[$cookieName]);
        session_start();
        
        //echo "<pre>";
        //print_r($_SESSION);
        //echo "</pre>";
        //die();

        // Check to make sure it exists and is not empty
        if (!isset($_SESSION[$usernameVar]) || $_SESSION[$usernameVar] === '') {
            if (CONST_AJAX) {
                throw new NagVisException(l('Nagios XI: Not authenticated.'));
            }
            header("Location: ../../../nagiosxi/login.php?redirect=".urlencode($url));
        }

        // Authenticate without providing logon information based on session value
        $username = $_SESSION[$usernameVar];
        $userlvl = $_SESSION[$userLevelVar];

        // Check userlevel to set perms
        $perms = 'Users (read-only)';
        if ($userlvl == 255) {
            $perms = 'Administrators';
        }

        if ($this->verifyUserExists($username, 1, $perms, $printErr) === false) {
            return;
        }

        $AUTH->setTrustUsername(true);
        $AUTH->setLogoutPossible(false);
        $AUTH->passCredentials(Array('user' => $username));
        return $AUTH->isAuthenticated();
   }
}
